@@ -1,3 +1,4 @@
+globalVariables("FUNCTION_TBD")
 # 
 # issues to deal with
 #   * var() is hybrid of 1-ary, 2-ary
@@ -17,10 +18,12 @@
 #' @aliases aggregatingFunction1 
 #' @param fun a function that takes a numeric vector and computes a summary statistic,
 #' returning a numeric vector of length 1.
-#' @param output.multiple a boolean indicating whether \code{..fun..} returns multiple values
-#' @param input.multiple a boolean indicating whether \code{..fun..} can accept 2 vectors (e.g., \code{var})
+#' @param output.multiple a boolean indicating whether \code{fun} returns multiple values
 #' @param envir an environment in which evaluation takes place.
 #' @param na.rm the default value for na.rm in the resulting function.
+#' @param style one of \code{"flexible"} or \code{"formula"}.  In the latter case, the first
+#' argument must be a formula or evaluate to an object.  In the former case, bare names will
+#' be converted into formulas.
 #' @return a function that generalizes \code{fun} to handle a formula/data frame interface.
 #' 
 #' @examples
@@ -31,23 +34,166 @@
 #'   foo( length ~ sex, data=KidsFeet )
 #' } 
 #' @export
-aggregatingFunction1 <- function( fun, input.multiple=FALSE, output.multiple=FALSE, 
-                                  envir=parent.frame(), na.rm=getOption("na.rm",FALSE) ) {
+# 
+
+
+# Testing a possible replacement for aggregatingFunction1
+# Might be masked below by the old one -- be sure to check.
+
+aggregatingFunction1 <- 
+  function( fun, 
+            output.multiple=FALSE, 
+            envir=parent.frame(), 
+            na.rm=getOption("na.rm",FALSE),
+            style = c("flexible", "formula")
+  ) {
+    fun <- deparse(substitute(fun))
+    style = match.arg(style)
+    templates <- list(
+      flexible = 
+      function(
+        x, ..., 
+        data = NULL,
+        groups = NULL, 
+        na.rm = getOption("na.rm", FALSE)) 
+      {
+        pframe <- parent.frame()
+        subst_x <- substitute(x)
+        lazy_formula <- 
+          tryCatch(
+            lazyeval::lazy(x),
+            error = function(e) {
+              if (grepl("Promise has already been forced", e$message) ||
+                  # next line can be deleted when lazyeval updates on CRAN
+                  grepl("e of NULL environment", e$message) 
+                  ) 
+                NULL
+              else 
+                stop(e)
+            }
+          )
+        if (is.null(lazy_formula)) {
+          lazy_formula <- structure(list(expr=subst_x, env=pframe), class="lazy")
+        }
+        if (is.null(data)) {
+          result <-
+            tryCatch(FUNCTION_TBD(x, ..., na.rm = na.rm), 
+                     error=function(e) {e} , 
+                     warning=function(w) {w} ) 
+          if (! inherits(result, c("error", "warning")))  return(result) 
+          data <- parent.frame()
+        }
+        formula <- formularise(lazy_formula, parent.frame(2)) 
+        formula <- mosaic_formula_q(formula, groups=groups, max.slots=3) 
+        maggregate(formula, data=data, FUN = FUNCTION_TBD, ..., .multiple = output.multiple)
+      },
+      formula = 
+      function(
+        x, ..., 
+        data = NULL,
+        groups = NULL, 
+        na.rm = getOption("na.rm", FALSE)) 
+      {
+        if (is.null(data)) {
+          result <-
+            tryCatch(FUNCTION_TBD(x, ..., na.rm = na.rm), error=function(e) {e} , warning=function(w) {w} ) 
+          if (! inherits(result, c("error", "warning")))  return(result) 
+          data <- parent.frame()
+        }
+        if (! inherits(x, "formula")) stop( "`x' must be a formula")
+        formula <- mosaic_formula_q(x, groups=groups, max.slots=3) 
+        maggregate(formula, data=data, FUN = FUNCTION_TBD, ..., .multiple = output.multiple)
+      }
+    )
+    
+    fun_text <- deparse(templates[[style]])
+    fun_text <- gsub("FUNCTION_TBD", fun, fun_text) 
+    fun_text <- gsub("output.multiple", output.multiple, fun_text)
+    res <- eval(parse( text = fun_text))
+    environment(res) <- parent.frame()
+    res
+  } 
+
+# Old version -- append a to name to use the new one above.
+aggregatingFunction1a <-
+  function (fun, input.multiple = FALSE, output.multiple = FALSE, 
+          envir = parent.frame(), na.rm = getOption("na.rm", FALSE)) 
+{
+  result <- function(x, ..., data, groups = NULL) {
+    orig.call <- match.call()
+    fun.call <- orig.call
+    fun.call[[1]] <- substitute(..fun..)
+    fun.call[[2]] <- substitute(x)
+    missingData <- FALSE
+    if (missing(data)) {
+      missingData <- TRUE
+      data = parent.frame()
+      result <- tryCatch(eval(fun.call, envir = parent.frame()), 
+                         error = function(e) {
+                           e
+                         }, warning = function(w) {
+                           w
+                         })
+      if (!inherits(result, "warning") && !inherits(result, 
+                                                    "error")) {
+        return(result)
+      }
+    }
+    maggregate.call <- orig.call
+    x_name <- substitute(x)
+    if (!.is.formula(x)) {
+      if (!missingData) {
+        fun.call[["data"]] <- NULL
+      }
+      if (input.multiple) {
+        result <- tryCatch(eval(fun.call, envir = data, 
+                                enclos = parent.frame()), error = function(e) {
+                                  e
+                                }, warning = function(w) {
+                                  w
+                                })
+        if (!inherits(result, "warning") && !inherits(result, 
+                                                      "error")) {
+          return(result)
+        }
+      }
+      x <- eval(substitute(mosaic_formula_q(.x, groups = quote(groups)), 
+                           list(.x = substitute(x), .g = substitute(groups))))
+      if ("groups" %in% names(maggregate.call)) 
+        maggregate.call[["groups"]] <- NULL
+    }
+    maggregate.call[[1]] <- quote(maggregate)
+    maggregate.call$formula <- x
+    maggregate.call$data <- data
+    maggregate.call$x <- NULL
+    maggregate.call$FUN <- substitute(..fun..)
+    maggregate.call$.multiple <- output.multiple
+    maggregate.call$na.rm <- substitute(na.rm)
+    return(eval(maggregate.call, envir = envir))
+  }
+  formals(result) <- c(formals(result), ..fun.. = substitute(fun), 
+                       na.rm = substitute(na.rm))
+  return(result)
+}
+# don't bother exporting this one (for now)
+
+aggregatingFunction1or2 <- function( fun, input.multiple=FALSE, output.multiple=FALSE, 
+                                     envir=parent.frame(), na.rm=getOption("na.rm",FALSE) ) {
   result <- function( x, ..., data, groups=NULL) {
     orig.call <- match.call()
     fun.call <- orig.call 
     fun.call[[1]] <- substitute(..fun..)
     fun.call[[2]] <- substitute(x)
-
+    
     # if data is not given, we will try to evaluate fun()
     missingData <- FALSE  
     if ( missing(data) ) {
       missingData <- TRUE
       data=parent.frame()
-
+      
       result <- tryCatch( eval(fun.call, envir=parent.frame()) , 
-                error=function(e) {e} ,
-                warning=function(w) {w} ) 
+                          error=function(e) {e} ,
+                          warning=function(w) {w} ) 
       if ( ! inherits(result, "warning") && ! inherits(result,"error") ) {
         return(result) 
       }
@@ -58,7 +204,7 @@ aggregatingFunction1 <- function( fun, input.multiple=FALSE, output.multiple=FAL
     x_name <- substitute(x)
     if (! .is.formula(x) ) {
       if ( !missingData) {
-         fun.call[['data']] <- NULL
+        fun.call[['data']] <- NULL
       }
       if (input.multiple) {
         result <- tryCatch( eval(fun.call, envir=data, enclos=parent.frame()),
@@ -75,7 +221,7 @@ aggregatingFunction1 <- function( fun, input.multiple=FALSE, output.multiple=FAL
       ) )
       
       if ("groups" %in% names(maggregate.call)) maggregate.call[['groups']] <- NULL
-    
+      
     }
     # now x is a formula
     
@@ -86,13 +232,14 @@ aggregatingFunction1 <- function( fun, input.multiple=FALSE, output.multiple=FAL
     maggregate.call$FUN <- substitute(..fun..)  # keep substitute here or no?
     maggregate.call$.multiple <- output.multiple
     maggregate.call$na.rm <- substitute(na.rm)
-#    print(maggregate.call)
+    #    print(maggregate.call)
     return( eval(maggregate.call, envir=envir) )
   }
   formals(result) <- c(formals(result), ..fun.. = substitute(fun), na.rm=substitute(na.rm))
   return(result)
 }
 
+
 #' 2-ary Aggregating functions
 #' 
 #' \code{aggregatinFuntion2} creates statistical summaries of two numerical vectors that are formula aware.
@@ -154,7 +301,6 @@ aggregatingFunction2 <- function( fun ) {
 #' @aliases sum min max mean median sd var cov cor favstats
 #' @param x an object, often a formula
 #' @param y an object, often a numeric vector 
-#' @param ..fun.. the underlyin function used in the computation
 #' @param groups a grouping variable, typically a name of a variable in \code{data}
 #' @param data a data frame in which to evaluate formulas (or bare names).
 #' Note that the default is \code{data=parent.frame()}.  This makes it convenient to
@@ -163,6 +309,8 @@ aggregatingFunction2 <- function( fun ) {
 #' When programming, it is best to use an explicit \code{data} argument
 #' -- ideally supplying a data frame that contains the variables mentioned.
 #' @param \dots additional arguments
+#' @param ..fun.. the underlying function.  It would be very unusual to change 
+#' this from its default value.
 #' @param na.rm a logical indicating whether \code{NA}s should be removed before computing
 #' @export
 mean <- aggregatingFunction1( base::mean )
@@ -210,7 +358,7 @@ quantile <- aggregatingFunction1(stats::quantile, output.multiple=TRUE,
                                  na.rm=getOption("na.rm", FALSE))
 #' @rdname aggregating
 #' @export
-var <- aggregatingFunction1( stats::var, input.multiple=TRUE )
+var <- aggregatingFunction1or2( stats::var, input.multiple=TRUE )
 #' @rdname aggregating
 #' @export
 cor <- aggregatingFunction2( stats::cor )
@@ -282,7 +430,6 @@ SAD_ <- function(x, ..., na.rm = getOption("na.omit", FALSE)) {
 #'   interpret the formala in a data frame.
 #' @param na.rm a logical indicating whether NAs should be removed before
 #'   calculaing.
-#' @param ..fun.. the underlying function used in the computation
 #' @param groups a grouping variable, typically a name of a variable in \code{data}
 #' @param data a data frame in which to evaluate formulas (or bare names).
 #'   Note that the default is \code{data=parent.frame()}.  This makes it convenient to
