@@ -271,19 +271,33 @@ mUSMap <- function(data, key, fill=NULL,
 #' available dataset. If this name is passed as an argument to the function, the function 
 #' will return the corresponding dataset.
 #' 
-#' @param name An optional parameter specifying the name of the desired dataset
+#' @param name An optional parameter specifying the name of the desired dataset.
+#' If multiple names are given, a merge will be attempted on the individual data
+#' sets.
 #' 
 #' @examples
 #' head(CIAdata())
-#' gdpData <- CIAdata("pop")
-#' nrow(gdpData)
+#' Population <- CIAdata("pop")
+#' nrow(Population)
+#' head(Population)
 #' 
-#' mergedData <- merge(CIAdata("pop"), CIAdata("fert"), by="country")
-#' head(mergedData)
+#' PopArea <- CIAdata(c("pop","area")) %>% mutate(density = pop / area)
+#' nrow(PopArea)
+#' head(PopArea)
+#' PopArea %>% 
+#'   filter(!is.na(density)) %>%
+#'   arrange(density) %>% 
+#'   tail
 #' @export
 CIAdata <- function (name = NULL) {
+  
   if (is.null(name)) return(CIA)  
   
+  if (length(name) > 1) {
+    return(Reduce(function(A,B) merge(A,B, by="country", all=TRUE), 
+                  Map(CIAdata, name=name)))
+  }
+  
   if (name %in% CIA$Name) {
     sub <- subset(CIA, Name == name)
   } else {
@@ -297,11 +311,17 @@ CIAdata <- function (name = NULL) {
   code <- sub[["Code"]]
   url <- (paste0("https://www.cia.gov/library/publications/the-world-factbook/rankorder/rawdata_",
                  code, ".txt"))
+  message(paste("Retrieving data from", url))
   
   if (! requireNamespace("RCurl")) stop("Package `RCurl' must be installed.")
+
+# it appears that the file format has moved from tab delimited to fixed width
+# parsing on two or more spaces seems to work, at least for some files.
   
-  table <- read.delim(textConnection(RCurl::getURL(url, ssl.verifypeer = FALSE)),
-                      header = FALSE, stringsAsFactors = FALSE)
+#  table <- read.delim(textConnection(RCurl::getURL(url, ssl.verifypeer = FALSE)),
+#                      header = FALSE, stringsAsFactors = FALSE)
+  lines <- readLines(textConnection(RCurl::getURL(url, ssl.verifypeer = FALSE)))
+  table <- as.data.frame(do.call(rbind, strsplit( lines, "  +")), stringsAsFactors=FALSE)
   table[, 1] <- NULL
   names(table) <- c("country", name)
   table[[2]] = as.numeric(gsub("[^.+[:digit:] ]", "",
