@@ -11,17 +11,15 @@
 #' @return a contingency table, an object of class "table", an array of percentage or proportion
 #' values. Note that unlike S the result is always an array, a 1D array if one factor is given.
 #' @examples
-#' perctable(rbinom(1000,10,.5))
-#' with(airquality,
-#'    perctable(OzHi=Ozone > 80, Month, useNA="ifany"))
-#' with(airquality,
-#'    perctable(OzHi=Ozone > 80, Month, useNA="always"))
+#' # perctable() has been deprecated.  Use tally() instead
+#' example(tally)
 #' 
 #' @keywords manipulate 
 #' @export
 
 perctable <- function(...) 
 {
+  .Deprecated("tally")
 	t <- table(...)
 	t/sum(t) * 100
 }
@@ -31,6 +29,7 @@ perctable <- function(...)
 
 proptable <- function(...) 
 {
+  .Deprecated("tally")
 	t <- table(...)
 	t/sum(t) 
 }
